(*:Mathematica Version: 5.0 *)

(*:Package Version: 1.1 *)

(*:Copyright: Copyright 2004, Wellin, Gaylord, and Kamin *)

(*:Title: IPM3`RandomWalks` *)

(* From the book Introduction to Programming with Mathematica, 3rd Ed. *)
   
(*:Author: Paul R. Wellin and Richard J. Gaylord, and Samuel N. Kamin *)

(*:Summary:
	The code is presented and discussed
	in the book Introduction to Programming with Mathematica,
	3rd Edition, Cambridge University Press, 2004. *)

(* ******************************************************** *)

BeginPackage["IPM3`RandomWalks`"]; 

Needs["Graphics`ArgColors`"]
Needs["Graphics`Polyhedra`"];

RandomWalk::"usage"="RandomWalk[n] generates an n-step walk in two dimensions. The default behavior gives a lattice walk with steps in one of the four compass directions. The option LatticeWalk takes values True or False. The value of the option Dimensions can be any of 1, 2, or 3.";

LatticeWalk::"usage"="LatticeWalk\[Rule]val is an option to RandomWalk that determines whether the random walk will be a lattice walk or an off-lattice walk. Possible values are True and False.";

ShowWalk::"usage"="ShowWalk[walk] displays a two-dimensional random walk connecting each site with a line. Any Graphics options can be passed to ShowWalk. E.g., ShowWalk[walk,  Background\[Rule]GrayLevel[0]] to produce a black background.";

AnimateWalk::"usage"="AnimateWalk[walk, opts] creates an animation of a two-dimensional random walk. A red ball will be seen to move to the current position in the walk to aid in visualizing the animation.";

SquareDistance::usage = 
"SquareDistance[walk] computes the \
square of the end-to-end distance of a two-dimensional lattice \
walk. This can be thought of as the square of the diameter of \
the walk.";

MeanSquareDistance::usage = 
"MeanSquareDistance[n, m] computes \
the mean square end-to-end distance of m n-step lattice walks.";

MeanSquareRadiusGyration::usage = 
"MeanSquareRadiusGyration[m, n] \
computes the mean square radius of gyration of a random walk. This \
is the sum of the squares of the distances of the step locations \
from the center of mass, divided by the number of step locations, \
where the center of mass is the sum of the step locations divided \
by the number of step locations.";

Options[RandomWalk] = {LatticeWalk -> True, Dimension -> 2}; 

Begin["`Private`"]; 

RandomWalk::rwn = "Argument `1` is not a positive integer."

walk1D[n_] := NestList[#1 + (-1)^Random[Integer] & , 0, n]

walk1DOffLattice[n_] := FoldList[Plus, 0, Table[Random[Real, {-1, 1}], {n}]]

walk2D[n_] := Module[{NSEW = {{0, 1}, {1, 0}, {0, -1}, {-1, 0}}}, 
   FoldList[Plus, {0, 0}, NSEW[[Table[Random[Integer, {1, 4}], {n}]]]]]

walk2DOffLattice[n_] := FoldList[Plus, {0, 0}, ({Cos[#1], Sin[#1]} & ) /@ 
    Table[Random[Real, {0, 2*Pi}], {n}]]

walk3D[n_] := Module[{NSEW3 = Sqrt[2]*Vertices[Cube]}, 
   FoldList[Plus, {0, 0, 0}, NSEW3[[Table[Random[Integer, {1, 8}], {n}]]]]]

walk3DOffLattice[n_] := FoldList[Plus, {0, 0, 0}, 
   ({Cos[#1], Sin[#1], #1/(2*Pi)} & ) /@ Table[Random[Real, {-2*Pi, 2*Pi}], 
     {n}]]

RandomWalk[n_, (opts___)?OptionQ] := Module[{dim, latticeQ}, 
   If[ !(IntegerQ[n] && n > 0), Message[RandomWalk::rwn, n], 
    {latticeQ, dim} = {LatticeWalk, Dimension} /. 
       Flatten[{opts, Options[RandomWalk]}]; Which[dim == 1, 
      If[latticeQ, walk1D[n], walk1DOffLattice[n]], dim == 2, 
      If[latticeQ, walk2D[n], walk2DOffLattice[n]], dim == 3, 
      If[latticeQ, walk3D[n], walk3DOffLattice[n]]]]]

RandomWalk[x__] /; Message[RandomWalk::rwn, x] := Null
RandomWalk[] /; Message[General::argx,RandomWalk,0]:=Null;

(* NUMERICAL ANALYSIS FUNCTIONS *)

SquareDistance[walk_List]:= 
	Apply[Plus, (Last[walk] - First[walk])^2];

MeanSquareDistance[n_Integer, m_Integer]:= 
	N[Sum[SquareDistance[ Walk2D[n] ], {m}]/m];

MeanSquareRadiusGyration[m_Integer, n_Integer]:= 
Module[{},
   squareRadiusOfGyration[s_Integer]:= 
   Module[{locs, cm, choices = {{1,0},{-1,0},{0,1},{0,-1}}},
      locs = FoldList[Plus,{0,0}, choices[[Table[Random[Integer,{1,4}],{s}]]]];
      cm = N[Apply[Plus, locs]/(s+1)];
      Apply[Plus, Flatten[(Transpose[locs] - cm)^2]]/(s+1)];
   N[Sum[squareRadiusOfGyration[n], {m}]/m]
];

(* VISUALIZATION FUNCTIONS *)

ShowWalk[coords_, opts___] := Which[Length[Dimensions[coords]] == 1, 
   ListPlot[coords, opts, PlotJoined -> True], Dimensions[coords][[2]] == 2, 
   Show[Graphics[Line[coords], opts, AspectRatio -> Automatic]], 
   Dimensions[coords][[2]] == 3, Show[Graphics3D[Line[coords], opts, 
     AspectRatio -> Automatic]]]

AnimateWalk[coords_, opts___] := 
  Scan[Show[Graphics[{{RGBColor[1, 0, 0], PointSize[0.025], 
        Point[coords[[#1]]]}, Line[Take[coords, #1]]}], opts, 
     AspectRatio -> Automatic, PlotRange -> 
      ({Min[#1] - 0.2, Max[#1] + 0.2} & ) /@ Transpose[coords]] & , 
   Range[2, Length[coords]]]

End[];

EndPackage[];
