(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     33503,       1157]*)
(*NotebookOutlinePosition[     43705,       1461]*)
(*  CellTagsIndexPosition[     43441,       1448]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Random Walks", "Section",
  CellTags->{"rw:1", "rw:2", "rw:3", "rw:4", "rw:5", "rw:6", "rw:7"}],

Cell[TextData[{
  "From the book, ",
  StyleBox["Introduction to Programming with Mathematica",
    FontSlant->"Italic"],
  ", 3rd Edition. \n\[Copyright] Cambridge University Press 2004"
}], "Text",
  CellFrame->True,
  Background->GrayLevel[0.833326]],

Cell[CellGroupData[{

Cell["Introduction", "Subsection",
  CellTags->"rw:1"],

Cell[TextData[{
  "The use of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to perform diverse computing tasks can be illustrated using the random \
walk model. This model, which can be envisioned by thinking of a person \
taking a succession of steps which are randomly oriented with respect to one \
another, is widely used to represent",
  " ",
  "random processes in nature; physicists model the transport of molecules, \
biologists model the locomotion of organisms, engineers model heat \
conduction, and economists model the time behavior of financial markets, all \
with the random walk model."
}], "Text",
  CellTags->"rw:1"],

Cell["\<\
In this section, we will develop a program for executing a random walk. Then \
we will run the program and create a visualization of the walk that is \
created. In the course of our application development, we discuss options, \
defaults, and documentation issues.\
\>", "Text",
  CellTags->"rw:1"]
}, Open  ]],

Cell[CellGroupData[{

Cell["The One-Dimensional Random Walk", "Subsection",
  CellTags->"rw:2"],

Cell[TextData[{
  "The simplest random walk model consists of ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " steps of equal length, back-and-forth along a line. A step increment (or \
step) in the positive ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  "-direction corresponds to a value of 1 and a step increment in the \
negative ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  "-direction corresponds to a value of ",
  Cell[BoxData[
      \(TraditionalForm\`\(-1\)\)]],
  ". A list of the successive step increments of an ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  "-step random walk in one dimension is therefore a list of ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " randomly selected 1s and ",
  Cell[BoxData[
      \(TraditionalForm\`\(-1\)\)]],
  "s. This list can be generated in many ways. Here is one straightforward \
implementation, generating a list of ten steps."
}], "Text",
  CellTags->"rw:2"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[\((\(-1\))\)\^Random[Integer], {10}]\)], "Input",
  CellTags->"rw:2"],

Cell[BoxData[
    \({1, 1, 1, 1, \(-1\), 1, \(-1\), \(-1\), 1, \(-1\)}\)], "Output",
  CellTags->"rw:2"]
}, Open  ]],

Cell[TextData[{
  "A list generated in this manner can be used to generate a list of the ",
  Cell[BoxData[
      \(TraditionalForm\`n + 1\)]],
  " locations of a one-dimensional ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  "-step walk which starts at the origin, using the folding operation."
}], "Text",
  CellTags->"rw:2"],

Cell[CellGroupData[{

Cell[BoxData[
    \(FoldList[Plus, 0, %]\)], "Input",
  CellTags->"rw:2"],

Cell[BoxData[
    \({0, 1, 2, 3, 4, 3, 4, 3, 2, 3, 2}\)], "Output",
  CellTags->"rw:2"]
}, Open  ]],

Cell[TextData[{
  "We can now write the program ",
  StyleBox["Walk1D", "MR"],
  " to generate a list of the step locations of an ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  "-step random walk, originating at the origin."
}], "Text",
  CellTags->"rw:2"],

Cell[BoxData[
    \(Walk1D[n_] := 
      FoldList[Plus, 0, Table[\((\(-1\))\)\^Random[Integer], {n}]]\)], "Input",\

  CellTags->"rw:2"],

Cell[TextData[{
  "Here is a ten-step one-dimensional random walk using this ",
  StyleBox["Walk1D", "MR"],
  " program."
}], "Text",
  CellTags->"rw:2"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Walk1D[10]\)], "Input",
  CellTags->"rw:2"],

Cell[BoxData[
    \({0, \(-1\), \(-2\), \(-3\), \(-4\), \(-3\), \(-4\), \(-5\), \(-4\), \
\(-5\), \(-4\)}\)], "Output",
  CellTags->"rw:2"]
}, Open  ]],

Cell["\<\
A list of the step locations can also be generated without first creating a \
list of the step increments, using the nesting operation.\
\>", "Text",
  CellTags->"rw:2"],

Cell[BoxData[
    \(Walk1D2[n_] := 
      NestList[# + \((\(-1\))\)\^Random[Integer] &, 0, n]\)], "Input",
  CellTags->"rw:2"],

Cell["This is just slightly faster than the previous approach.", "Text",
  CellTags->"rw:2"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(n = 10\^6;\)\), "\[IndentingNewLine]", 
    \({Timing[\(Walk1D[n];\)], Timing[\(Walk1D2[n];\)]}\)}], "Input",
  CellTags->"rw:2"],

Cell[BoxData[
    \({{0.5399999999999999`\ Second, Null}, {0.401`\ Second, 
        Null}}\)], "Output",
  CellTags->"rw:2"]
}, Open  ]],

Cell[TextData[{
  "Finally, we can plot the random walk using ",
  StyleBox["ListPlot", "MR"],
  "."
}], "Text",
  CellTags->"rw:2"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[Walk1D[1000], PlotJoined \[Rule] True]\)], "Input",
  CellTags->"rw:2"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellTags->"rw:2"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Two-Dimensional Lattice Walk", "Subsection",
  CellTags->"rw:3"],

Cell["\<\
The random walk model in two or more dimensions is more complicated than the \
random walk in one dimension. Although each step of a one-dimensional walk is \
at 0 degrees (forward) or 180 degrees (backward) with respect to the \
preceding step, in higher dimensions each step can take a range of \
orientations with respect to the previous step.\
\>", "Text",
  CellTags->"rw:3"],

Cell["\<\
We\[CloseCurlyQuote]ll first consider a random walk on a lattice. \
Specifically, we\[CloseCurlyQuote]ll look at a lattice walk on the \
two-dimensional square lattice. This walk consists of steps of uniform \
length, randomly taken in the North, East, South, or West direction. The list \
of the possible step increments in this walk is given by the compass \
directions.\
\>", "Text",
  CellTags->"rw:3"],

Cell[CellGroupData[{

Cell[BoxData[
    \(NSEW = {{0, 1}, {1, 0}, {0, \(-1\)}, {\(-1\), 0}}\)], "Input",
  CellTags->"rw:3"],

Cell[BoxData[
    \({{0, 1}, {1, 0}, {0, \(-1\)}, {\(-1\), 0}}\)], "Output",
  CellTags->"rw:3"]
}, Open  ]],

Cell[TextData[{
  "Here is a list of ",
  Cell[BoxData[
      \(TraditionalForm\`5\)]],
  " step increments."
}], "Text",
  CellTags->"rw:3"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(n = 5;\)\), "\[IndentingNewLine]", 
    \(NSEW\[LeftDoubleBracket]
      Table[Random[Integer, {1, 4}], {n}]\[RightDoubleBracket]\)}], "Input",
  CellTags->"rw:3"],

Cell[BoxData[
    \({{0, 1}, {\(-1\), 0}, {0, \(-1\)}, {0, 1}, {0, 1}}\)], "Output",
  CellTags->"rw:3"]
}, Open  ]],

Cell[TextData[{
  "Here then is a program, called ",
  StyleBox["Walk2D", "MR"],
  " that generates a list of the step locations of an ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  "-step lattice walk starting at the origin {0, 0}."
}], "Text",
  CellTags->"rw:3"],

Cell[BoxData[
    \(Walk2D[
        n_] := \[IndentingNewLine]Module[{NSEW = {{0, 1}, {1, 
                0}, {0, \(-1\)}, {\(-1\), 0}}}, \[IndentingNewLine]FoldList[
          Plus, {0, 0}, \[IndentingNewLine]NSEW\[LeftDoubleBracket]
            Table[Random[
                Integer, {1, 4}], {n}]\[RightDoubleBracket]]]\)], "Input",
  CellTags->"rw:3"],

Cell["Here is a ten-step lattice walk in two dimensions.", "Text",
  CellTags->"rw:3"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Walk2D[10]\)], "Input",
  CellTags->"rw:3"],

Cell[BoxData[
    \({{0, 0}, {0, 1}, {0, 0}, {1, 
        0}, {1, \(-1\)}, {2, \(-1\)}, {3, \(-1\)}, {3, \(-2\)}, {4, \(-2\)}, \
{4, \(-3\)}, {4, \(-2\)}}\)], "Output",
  CellTags->"rw:3"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Visualizing The Two-Dimensional Lattice Walk", "Subsection",
  CellTags->"rw:4"],

Cell[TextData[{
  "We will create a snapshot of the path of the lattice walk using the \
graphics primitive ",
  StyleBox["Line", "MR"],
  " to draw lines between successive points in the walk."
}], "Text",
  CellTags->"rw:4"],

Cell[BoxData[
    \(ShowWalk2D[coords_, opts___] := 
      Show[Graphics[Line[coords], opts, 
          AspectRatio \[Rule] Automatic]]\)], "Input",
  CellTags->"rw:4"],

Cell[TextData[{
  "Here we have set the value of the ",
  StyleBox["AspectRatio", "MR"],
  " option to ",
  StyleBox["Automatic", "MR"],
  " so that steps in the ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  "- and ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  "-directions will appear to have equal lengths in the plot. This option can \
be overwritten by specifying a different value in the list of options given \
by ",
  StyleBox["opts", "MR"],
  ". Note the use of the triple blank in the definition of ",
  StyleBox["ShowWalk2D", "MR"],
  ". The pattern ",
  StyleBox["opts___", "MR"],
  " ",
  "matches any sequence (possibly empty) of rules which are used here to \
govern the display of the graphic by changing certain options to the ",
  StyleBox["Graphics", "MR"],
  " function. It is important that ",
  StyleBox["opts", "MR"],
  " appears ",
  StyleBox["before",
    FontSlant->"Italic"],
  " the option ",
  StyleBox["AspectRatio", "MR"],
  ". This will allow you to override this (or any) option value. If ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " sees an option listed more than once in a list of options, it will only \
use the first such option. If ",
  StyleBox["opts", "MR"],
  " had come at the end of this function, you would not be able to change the \
value for ",
  StyleBox["AspectRatio", "MR"],
  "."
}], "Text",
  CellTags->"rw:4"],

Cell["Here is a simple example of a two-dimensional lattice walk.", "Text",
  CellTags->"rw:4"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ShowWalk2D[Walk2D[20]]\)], "Input",
  CellTags->"rw:4"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellTags->"rw:4"]
}, Open  ]],

Cell["\<\
A lattice walk repeatedly revisits sites that have been previously visited in \
the course of its meandering. As a result, it is difficult, and usually \
impossible, to discern the history of the walk from a snapshot of the path. \
The best way to see the entire evolution process of the walk in an \
unobstructed fashion is to create an animation.\
\>", "Text",
  CellTags->"rw:4"],

Cell[TextData[{
  "Creating an animation of a random walk in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is straightforward. This can be explained using a short lattice walk as \
an example."
}], "Text",
  CellTags->"rw:4"],

Cell[CellGroupData[{

Cell[BoxData[
    \(walk = Walk2D[10]\)], "Input",
  CellTags->"rw:4"],

Cell[BoxData[
    \({{0, 0}, {0, 1}, {1, 1}, {1, 
        0}, {1, \(-1\)}, {2, \(-1\)}, {2, \(-2\)}, {2, \(-3\)}, {3, \(-3\)}, \
{3, \(-2\)}, {2, \(-2\)}}\)], "Output",
  CellTags->"rw:4"]
}, Open  ]],

Cell[TextData[{
  "The animation consists of a sequence of graphics cells where the first \
cell shows the first step of the walk (consisting of a line drawn between the \
first two elements in ",
  StyleBox["walk", "MR"],
  " for example) and each succeeding cell shows one more step than the \
previous cell. In general then, the ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  "th cell is drawn using the ",
  StyleBox["Line", "MR"],
  " function and the first ",
  Cell[BoxData[
      \(TraditionalForm\`m + 1\)]],
  " elements in ",
  StyleBox["walk", "MR"],
  ". All of the graphics cells are drawn by mapping ",
  StyleBox["Show", "MR"],
  " across the ",
  StyleBox["walk", "MR"],
  " list."
}], "Text",
  CellTags->"rw:4"],

Cell["\<\
Map[(Show[Graphics[Line[Take[walk, #]]]])&, 
    Range[2, Length[walk] ]]\
\>", "Program",
  CellTags->"rw:4"],

Cell["\<\
In general, objects in a graphics cell are scaled to fill the monitor screen. \
Therefore, if we simply create cells, each containing a different number of \
steps of the walk using the above graphics command, steps in one cell will \
appear to be of a different length then the same steps in other cells. This \
will result in a jerky-looking animation.\
\>", "Text",
  CellTags->"rw:4"],

Cell[TextData[{
  "We can make all of the step lengths in all of the graphics cells uniform \
by using the ",
  StyleBox["PlotRange", "MR"],
  " option with the ordered pair of the minimum and maximum values of the \
components of the random walk in each direction, ",
  StyleBox["{{", "MR"],
  Cell[BoxData[
      FormBox[
        StyleBox["xmin",
          FontSlant->"Italic"], TraditionalForm]]],
  StyleBox[",", "MR"],
  Cell[BoxData[
      FormBox[
        StyleBox["xmax",
          FontSlant->"Italic"], TraditionalForm]]],
  StyleBox["},{", "MR"],
  Cell[BoxData[
      FormBox[
        StyleBox["ymin",
          FontSlant->"Italic"], TraditionalForm]]],
  StyleBox[",", "MR"],
  Cell[BoxData[
      FormBox[
        StyleBox["ymax",
          FontSlant->"Italic"], TraditionalForm]]],
  StyleBox["}}", "MR"],
  ". This quantity can be determined by separating the ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  "- and ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  "-components of the walk using ",
  StyleBox["Transpose", "MR"],
  " and then mapping an anonymous function containing ",
  StyleBox["Min", "MR"],
  " and ",
  StyleBox["Max", "MR"],
  " onto it."
}], "Text",
  CellTags->"rw:4"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[{Min[#], Max[#]} &, Transpose[walk]]\)], "Input",
  CellTags->"rw:4"],

Cell[BoxData[
    \({{0, 3}, {\(-3\), 1}}\)], "Output",
  CellTags->"rw:4"]
}, Open  ]],

Cell["\<\
Here then, is the overall program for creating the animation.\
\>", "Text",
  CellTags->"rw:4"],

Cell[BoxData[
    \(AnimateWalk2D[coords_, opts___] := 
      Map[Show[Graphics[Line[Take[coords, #]]], opts, 
            AspectRatio \[Rule] Automatic, 
            PlotRange \[Rule] 
              Map[{Min[#] - 1, Max[#] + 1} &, Transpose[coords]]] &, 
        Range[2, Length[coords]]]\)], "Input",
  CellTags->"rw:4"],

Cell[TextData[{
  "Note: We have added 1 to the maximum ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  " values and subtracted 1 from the minimum ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  " values in order to enhance the display by making the graphics a little \
smaller inside its bounding box. Also note that we have replaced ",
  StyleBox["Map[Show[\[Ellipsis]]]", "MR"],
  " with ",
  StyleBox["Scan[Show[\[Ellipsis]]]", "MR"],
  ". ",
  StyleBox["Scan", "MR"],
  " is quite similar to ",
  StyleBox["Map", "MR"],
  " but its main difference is that it does not return an expression, so the \
",
  StyleBox["Show", "MR"],
  " is essentially a side effect of this computation. "
}], "Text",
  CellTags->"rw:4"],

Cell["\<\
While we can\[CloseCurlyQuote]t see the random walk animation run in a book, \
we can look at the graphics cells in the animation by creating a graphics \
array.\
\>", "Text",
  CellTags->"rw:4"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[
      GraphicsArray[
        Partition[AnimateWalk2D[walk, DisplayFunction \[Rule] Identity], 
          5]]]\)], "Input",
  CellTags->"rw:4"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  GraphicsArray  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellTags->"rw:4"]
}, Open  ]],

Cell[TextData[{
  "The option ",
  StyleBox["DisplayFunction\[Rule]Identity", "MR"],
  " is used to suppress the display of the individual graphics cells created \
by the ",
  StyleBox["AnimateWalk", "MR"],
  " function (the ",
  StyleBox["GraphicsArray", "MR"],
  " has its own ",
  StyleBox["DisplayFunction", "MR"],
  " function option whose default value is ",
  StyleBox["$DisplayFunction", "MR"],
  ") and the ",
  StyleBox["Partition", "MR"],
  " function is used to specify the number of graphics in each row of the ",
  StyleBox["GraphicsArray", "MR"],
  " picture."
}], "Text",
  CellTags->"rw:4"]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Three-Dimensional Random Walk", "Subsection",
  CellTags->"rw:5"],

Cell[TextData[{
  "For a three-dimensional random lattice walk, we will use the vertices of a \
cube as our directional vectors. We could input them manually, but they are \
defined in the ",
  StyleBox["Graphics`Polyhedra`", "MR"],
  " package so we may as well use those."
}], "Text",
  CellTags->"rw:5"],

Cell["<<Graphics`Polyhedra`", "Input",
  CellTags->"rw:5"],

Cell[CellGroupData[{

Cell[BoxData[
    \(NSEW3 = Vertices[Cube]\)], "Input",
  CellTags->"rw:5"],

Cell[BoxData[
    \({{1\/\@2, 1\/\@2, 1\/\@2}, {\(-\(1\/\@2\)\), 1\/\@2, 
        1\/\@2}, {\(-\(1\/\@2\)\), \(-\(1\/\@2\)\), 
        1\/\@2}, {1\/\@2, \(-\(1\/\@2\)\), 
        1\/\@2}, {\(-\(1\/\@2\)\), \(-\(1\/\@2\)\), \(-\(1\/\@2\)\)}, \
{1\/\@2, \(-\(1\/\@2\)\), \(-\(1\/\@2\)\)}, {1\/\@2, 
        1\/\@2, \(-\(1\/\@2\)\)}, {\(-\(1\/\@2\)\), 
        1\/\@2, \(-\(1\/\@2\)\)}}\)], "Output",
  CellTags->"rw:5"]
}, Open  ]],

Cell["Here then is the lattice walk in three dimensions.", "Text",
  CellTags->"rw:5"],

Cell[BoxData[
    \(Walk3D[n_] := 
      FoldList[Plus, {0, 0, 0}, 
        NSEW3\[LeftDoubleBracket]Table[
            Random[Integer, {1, 8}], {n}]\[RightDoubleBracket]]\)], "Input",
  CellTags->"rw:5"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Walk3D[5]\)], "Input",
  CellTags->"rw:5"],

Cell[BoxData[
    \({{0, 0, 0}, {\(-\(1\/\@2\)\), 1\/\@2, 1\/\@2}, {0, 
        0, \@2}, {1\/\@2, \(-\(1\/\@2\)\), 1\/\@2 + \@2}, {\@2, 0, 
        2\ \@2}, {\(-\(1\/\@2\)\) + \@2, 1\/\@2, 
        1\/\@2 + 2\ \@2}}\)], "Output",
  CellTags->"rw:5"]
}, Open  ]],

Cell[TextData[{
  "We can visualize this with only a slight modification to our earlier ",
  StyleBox["ShowWalk2D", "MR"],
  " function."
}], "Text",
  CellTags->"rw:5"],

Cell[BoxData[
    \(ShowWalk3D[coords_, opts___] := 
      Show[Graphics3D[Line[coords], opts, 
          AspectRatio \[Rule] Automatic]]\)], "Input",
  CellTags->"rw:5"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ShowWalk3D[Walk3D[1200]]\)], "Input",
  CellTags->"rw:5"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellTags->"rw:5"]
}, Open  ]],

Cell["Finally, we create an off-lattice walk in three dimensions.", "Text",
  CellTags->"rw:5"],

Cell[BoxData[
    \(OffLattice3D[n_] := 
      FoldList[Plus, {0, 0, 0}, 
        Map[{Cos[#], Sin[#], \(\(\ \)\(#\)\)\/\(2  \[Pi]\)} &, 
          Table[Random[Real, {\(-2\) \[Pi], 2\ \[Pi]}], {n}]]]\)], "Input",
  CellTags->"rw:5"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ShowWalk3D[OffLattice3D[1000]]\)], "Input",
  CellTags->"rw:5"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellTags->"rw:5"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Adding Options and Defaults", "Subsection",
  CellTags->"rw:6"],

Cell["\<\
When writing your own programs, it is often difficult to predict how a user \
will want to use some of your functions. Programmers usually try to provide a \
variety of ways to use their functions (allowing for different types of \
input, for example), or sometimes they write separate functions to handle \
special cases. The problem with having a separate function for each special \
case is that the user can soon become overloaded with the variety of \
functions to learn.\
\>", "Text",
  CellTags->"rw:6"],

Cell[TextData[{
  "In this section, we will show how to write options for your functions so \
that they behave like the built-in options in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". (In Chapter 12 we will add some error-trapping and messaging and we will \
see how to incorporate the use of options into a full-fledged package.) "
}], "Text",
  CellTags->"rw:6"],

Cell[TextData[{
  "The use of options and defaults in your programs allows you to minimize \
the use of many parameters and function names for the user to remember. For \
example, the built-in function ",
  StyleBox["FactorInteger", "MR"],
  " has an option ",
  StyleBox["GaussianIntegers", "MR"],
  " which, when set to ",
  StyleBox["True", "MR"],
  ", will factor a number over the gaussian integers."
}], "Text",
  CellTags->"rw:6"],

Cell[CellGroupData[{

Cell[BoxData[
    \(FactorInteger[5, GaussianIntegers \[Rule] True]\)], "Input",
  CellTags->"rw:6"],

Cell[BoxData[
    \({{\(-\[ImaginaryI]\), 1}, {1 + 2\ \[ImaginaryI], 
        1}, {2 + \[ImaginaryI], 1}}\)], "Output",
  CellTags->"rw:6"]
}, Open  ]],

Cell[TextData[{
  "The alternative to such an option would be to have a separate function, \
say ",
  StyleBox["FactorGaussianInteger", "MR"],
  ", that the user would have to use. Since the main process here is \
factorization of numbers, it makes sense to have one function that covers \
various situations allowing for factorization over different domains by \
specifying different options. "
}], "Text",
  CellTags->"rw:6"],

Cell["\<\
On the other hand, polynomial factorization is a fundamentally different \
operation from integer factorization, and so a different function is used for \
that.\
\>", "Text",
  CellTags->"rw:6"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Factor[
      27\ x\^5 + 81\ x\^4\ y + 9\ x\^3\ y\^2 - 73\ x\^2\ y\^3 + 32\ x\ y\^4 - 
        4\ y\^5]\)], "Input",
  CellTags->"rw:6"],

Cell[BoxData[
    \(\((3\ x - y)\)\^3\ \((x + 2\ y)\)\^2\)], "Output",
  CellTags->"rw:6"]
}, Open  ]],

Cell[TextData[{
  "In the previous sections we developed three separate functions, ",
  StyleBox["Walk1D", "MR"],
  ", ",
  StyleBox["Walk2D", "MR"],
  ", and ",
  StyleBox["Walk3D", "MR"],
  " that each generated random walks, the only difference being in the \
dimension of the walk. Since these functions are essentially the same, it \
would be better to combine them into one function, ",
  StyleBox["RandomWalk", "MR"],
  ", and set the dimension of the walk through the use of options. "
}], "Text",
  CellTags->"rw:6"],

Cell[TextData[{
  "We will define two options to ",
  StyleBox["RandomWalk", "MR"],
  ", ",
  StyleBox["Dimension", "MR"],
  " and ",
  StyleBox["LatticeWalk", "MR"],
  ". The ",
  StyleBox["LatticeWalk", "MR"],
  " option will be specified as a rule and when set to ",
  StyleBox["True", "MR"],
  ", will generate a lattice walk; when set to ",
  StyleBox["False", "MR"],
  ", it will generate an off-lattice walk. The following definition both \
defines options for the ",
  StyleBox["RandomWalk", "MR"],
  " function and specifies their default values."
}], "Text",
  CellTags->"rw:6"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Options[RandomWalk] = {LatticeWalk \[Rule] True, 
        Dimension \[Rule] 2}\)], "Input",
  CellTags->"rw:6"],

Cell[BoxData[
    \({LatticeWalk \[Rule] True, Dimension \[Rule] 2}\)], "Output",
  CellTags->"rw:6"]
}, Open  ]],

Cell[TextData[{
  "If you were now to ask for information about the ",
  StyleBox["RandomWalk", "MR"],
  " function, you would see these new options listed."
}], "Text",
  CellTags->"rw:6"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?RandomWalk\)\)], "Input",
  CellTags->"rw:6"],

Cell["Global`RandomWalk", "Print",
  CellTags->"Info3287377721-8917883"],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {GridBox[{
                {\(Options[RandomWalk] = {LatticeWalk \[Rule] True, 
                      Dimension \[Rule] 2}\)}
                },
              GridBaseline->{Baseline, {1, 1}},
              ColumnWidths->0.999,
              ColumnAlignments->{Left}]}
          },
        GridBaseline->{Baseline, {1, 1}},
        ColumnAlignments->{Left}],
      Definition[ "RandomWalk"],
      Editable->False]], "Print",
  CellTags->"Info3287377721-8917883"]
}, Open  ]],

Cell[TextData[{
  "As far as the ",
  StyleBox["LatticeWalk", "MR"],
  " option is concerned, we will use this option in the ",
  StyleBox["RandomWalk", "MR"],
  " function by branching to either a lattice walk or an off-lattice walk, \
depending upon the value of this new option. We will need to extract the \
value of this option inside the ",
  StyleBox["RandomWalk", "MR"],
  " function, and we can do this as follows:"
}], "Text",
  CellTags->"rw:6"],

Cell["\<\
latticeQ = \
LatticeWalk/.Flatten[\[InvisibleSpace]{opts,\[InvisibleSpace]Options[\
RandomWalk]}];\
\>", "Program",
  Evaluatable->False,
  CellTags->"rw:6"],

Cell[TextData[{
  "From right to left, the values of the options to ",
  StyleBox["RandomWalk", "MR"],
  " are substituted into ",
  StyleBox["opts", "MR"],
  "; then these (rules) are substituted to extract the value of ",
  StyleBox["LatticeWalk", "MR"],
  ". This value is then assigned to the symbol ",
  StyleBox["latticeQ", "MR"],
  ". "
}], "Text",
  CellTags->"rw:6"],

Cell[TextData[{
  "Similarly, we will extract the value of the option ",
  StyleBox["Dimension", "MR"],
  ". But we want to use the definitions given in the previous sections to \
branch appropriately depending upon the value of ",
  StyleBox["Dimension", "MR"],
  "; that is, depending upon whether we wish a 1D, 2D, or 3D random walk. The \
",
  StyleBox["Which", "MR"],
  " function is perfect for this task. "
}], "Text",
  CellTags->"rw:6"],

Cell[TextData[{
  "Which[\n\tdim == 1, ",
  StyleBox["use Walk1D definition",
    FontSlant->"Italic"],
  ",\n\tdim == 2, ",
  StyleBox["use Walk2D definition",
    FontSlant->"Italic"],
  ",\n\tdim == 3, ",
  StyleBox["use Walk3D definition",
    FontSlant->"Italic"],
  "]"
}], "Program",
  CellTags->"rw:6"],

Cell[TextData[{
  "Here then is the full function ",
  StyleBox["RandomWalk", "MR"],
  ", using this option structure."
}], "Text",
  CellTags->"rw:6"],

Cell["<<Graphics`Polyhedra`", "Input",
  CellTags->"rw:6"],

Cell[BoxData[
    \(\(RandomWalk[n_Integer, opts___?OptionQ] := 
        Module[{dim, latticeQ, NSEW, NSEW3, rand}, \[IndentingNewLine]rand := 
            Table[Random[
                Integer, {1, 2^dim}], {n}]; \[IndentingNewLine]NSEW = {{0, 
                1}, {1, 0}, {0, \(-1\)}, {\(-1\), 
                0}}; \[IndentingNewLine]NSEW3 = \(\@\(\(2\)\(\ \)\)\) 
              Vertices[Cube]; {latticeQ, dim} = {LatticeWalk, Dimension} /. 
              Flatten[{opts, 
                  Options[
                    RandomWalk]}]; \[IndentingNewLine]Which[\
\[IndentingNewLine]dim == 1, 
            FoldList[Plus, 0, 
              Table[\((\(-1\))\)\^Random[Integer], {n}]], \
\[IndentingNewLine]dim == 2, 
            If[latticeQ, FoldList[Plus, {0, 0}, NSEW[\([rand]\)]], 
              FoldList[Plus, {0, 0}, 
                Map[{Sin[#], Cos[#]} &, 
                  Table[Random[
                      Real, {0, 2\ \[Pi]}], {n}]]]], \[IndentingNewLine]dim == 
              3, If[latticeQ, 
              FoldList[Plus, {0, 0, 0}, 
                NSEW3[\([rand]\)]], \[IndentingNewLine]FoldList[
                Plus, {0, 0, 0}, 
                Map[{Cos[#], Sin[#], #\/\(2  \[Pi]\)} &, 
                  Table[Random[
                      Real, {\(-2\) \[Pi], 
                        2\ \[Pi]}], {n}]]]]\[IndentingNewLine]]\
\[IndentingNewLine]];\)\)], "Input",
  CellTags->"rw:6"],

Cell[TextData[{
  "Notice that if the ",
  StyleBox["LatticeWalk", "MR"],
  " option has been set to ",
  StyleBox["True", "MR"],
  ", then the first branch of the ",
  StyleBox["If", "MR"],
  " statement is followed, giving the lattice walk. If ",
  StyleBox["LatticeWalk", "MR"],
  " has any other value (",
  StyleBox["False", "MR"],
  " for example), then the off-lattice definition is used."
}], "Text",
  CellTags->"rw:6"],

Cell[TextData[{
  "This uses the default value of ",
  StyleBox["LatticeWalk", "MR"],
  ". and the default value of ",
  StyleBox["Dimension", "MR"],
  " (2)."
}], "Text",
  CellTags->"rw:6"],

Cell[CellGroupData[{

Cell[BoxData[
    \(RandomWalk[5]\)], "Input",
  CellTags->"rw:6"],

Cell[BoxData[
    \({{0, 0}, {0, \(-1\)}, {0, 0}, {1, 
        0}, {1, \(-1\)}, {0, \(-1\)}}\)], "Output",
  CellTags->"rw:6"]
}, Open  ]],

Cell["This creates an off-lattice walk.", "Text",
  CellTags->"rw:6"],

Cell[CellGroupData[{

Cell[BoxData[
    \(RandomWalk[4, LatticeWalk \[Rule] False]\)], "Input",
  CellTags->"rw:6"],

Cell[BoxData[
    \({{0, 
        0}, {\(-0.3048947667820382`\), \(-0.95238604630104`\)}, \
{\(-0.7494650863252059`\), \(-1.8481300002193937`\)}, {0.1077282137726624`, \
\(-2.3631248023944504`\)}, {\(-0.8637292628989264`\), \
\(-2.6003385688981453`\)}}\)], "Output",
  CellTags->"rw:6"]
}, Open  ]],

Cell["Here is a three-dimensional walk.", "Text",
  CellTags->"rw:6"],

Cell[CellGroupData[{

Cell[BoxData[
    \(RandomWalk[4, Dimension \[Rule] 3]\)], "Input",
  CellTags->"rw:6"],

Cell[BoxData[
    \({{0, 0, 0}, {1, 1, \(-1\)}, {2, 0, 
        0}, {3, \(-1\), \(-1\)}, {4, \(-2\), \(-2\)}}\)], "Output",
  CellTags->"rw:6"]
}, Open  ]],

Cell[TextData[{
  "Just as we have combined our various random walks into one function, so \
should we combine the functions to visualize these walks, using ",
  StyleBox["Which", "MR"],
  " to determine which branch to take."
}], "Text",
  CellTags->"rw:6"],

Cell[BoxData[
    \(ShowWalk[coords_, opts___] := 
      Which[\[IndentingNewLine]\(Dimensions[
              coords]\)\[LeftDoubleBracket]2\[RightDoubleBracket] == 2, 
        Show[Graphics[Line[coords], opts, 
            AspectRatio \[Rule] Automatic]], \(Dimensions[
              coords]\)\[LeftDoubleBracket]2\[RightDoubleBracket] == 3, 
        Show[Graphics3D[Line[coords], opts, 
            AspectRatio \[Rule] Automatic]]]\)], "Input",
  CellTags->"rw:6"],

Cell["Here then are several examples of these functions.", "Text",
  CellTags->"rw:6"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ShowWalk[
      RandomWalk[10\^4, Dimension \[Rule] 2, 
        LatticeWalk \[Rule] False]]\)], "Input",
  CellTags->"rw:6"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellTags->"rw:6"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ShowWalk[
      RandomWalk[10\^3, Dimension \[Rule] 3, 
        LatticeWalk \[Rule] True]]\)], "Input",
  CellTags->"rw:6"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellTags->"rw:6"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ShowWalk[
      RandomWalk[10\^4, 
        Dimension \[Rule] 2], \[IndentingNewLine]Frame \[Rule] True, 
      Background \[Rule] GrayLevel[0.9]]\)], "Input",
  CellTags->"rw:6"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellTags->"rw:6"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Creating Help Browser Documentation", "Subsection",
  CellTags->"rw:7"],

Cell["\<\
BrowserCategory[\"Random Walks\", None, {
\tItem[\"Introduction\", \"file.nb\", CopyTag->\"xxx\"],
\tItem[\"The One-Dimensional Random Walk\", \"file.nb\", CopyTag->\"xxx\"],
\tItem[\"The Two-Dimensional Lattice Walk\", \"file.nb\", CopyTag->\"xxx\"],
\tItem[\"Visualizing the Lattice Walk\", \"file.nb\", CopyTag->\"xxx\"],
\tItem[\"The Three-Dimensional Random Walk\", \"file.nb\", CopyTag->\"xxx\"],
\tItem[\"Adding Options and Defaults\", \"file.nb\", CopyTag->\"xxx\"],
\tItem[\"Creating Help Browser Documentation\", \"file.nb\", \
CopyTag->\"xxx\"]
\t}]\
\>", "Program",
  CellTags->"rw:7"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 685}},
WindowSize->{642, 575},
WindowMargins->{{44, Automatic}, {Automatic, 2}},
StyleDefinitions -> "Default.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "rw:1"->{
    Cell[1776, 53, 101, 1, 73, "Section",
      CellTags->{"rw:1", "rw:2", "rw:3", "rw:4", "rw:5", "rw:6", "rw:7"}],
    Cell[2158, 67, 54, 1, 38, "Subsection",
      CellTags->"rw:1"],
    Cell[2215, 70, 647, 14, 109, "Text",
      CellTags->"rw:1"],
    Cell[2865, 86, 308, 6, 71, "Text",
      CellTags->"rw:1"]},
  "rw:2"->{
    Cell[1776, 53, 101, 1, 73, "Section",
      CellTags->{"rw:1", "rw:2", "rw:3", "rw:4", "rw:5", "rw:6", "rw:7"}],
    Cell[3210, 97, 73, 1, 38, "Subsection",
      CellTags->"rw:2"],
    Cell[3286, 100, 950, 27, 109, "Text",
      CellTags->"rw:2"],
    Cell[4261, 131, 95, 2, 31, "Input",
      CellTags->"rw:2"],
    Cell[4359, 135, 104, 2, 29, "Output",
      CellTags->"rw:2"],
    Cell[4478, 140, 333, 9, 52, "Text",
      CellTags->"rw:2"],
    Cell[4836, 153, 73, 2, 30, "Input",
      CellTags->"rw:2"],
    Cell[4912, 157, 87, 2, 29, "Output",
      CellTags->"rw:2"],
    Cell[5014, 162, 262, 8, 52, "Text",
      CellTags->"rw:2"],
    Cell[5279, 172, 136, 4, 31, "Input",
      CellTags->"rw:2"],
    Cell[5418, 178, 153, 5, 33, "Text",
      CellTags->"rw:2"],
    Cell[5596, 187, 63, 2, 30, "Input",
      CellTags->"rw:2"],
    Cell[5662, 191, 139, 3, 29, "Output",
      CellTags->"rw:2"],
    Cell[5816, 197, 179, 4, 52, "Text",
      CellTags->"rw:2"],
    Cell[5998, 203, 126, 3, 31, "Input",
      CellTags->"rw:2"],
    Cell[6127, 208, 92, 1, 33, "Text",
      CellTags->"rw:2"],
    Cell[6244, 213, 152, 3, 50, "Input",
      CellTags->"rw:2"],
    Cell[6399, 218, 124, 3, 29, "Output",
      CellTags->"rw:2"],
    Cell[6538, 224, 132, 5, 33, "Text",
      CellTags->"rw:2"],
    Cell[6695, 233, 100, 2, 30, "Input",
      CellTags->"rw:2"],
    Cell[6798, 237, 150, 4, 29, "Output",
      CellTags->"rw:2"]},
  "rw:3"->{
    Cell[1776, 53, 101, 1, 73, "Section",
      CellTags->{"rw:1", "rw:2", "rw:3", "rw:4", "rw:5", "rw:6", "rw:7"}],
    Cell[6997, 247, 74, 1, 38, "Subsection",
      CellTags->"rw:3"],
    Cell[7074, 250, 390, 7, 90, "Text",
      CellTags->"rw:3"],
    Cell[7467, 259, 416, 8, 90, "Text",
      CellTags->"rw:3"],
    Cell[7908, 271, 102, 2, 30, "Input",
      CellTags->"rw:3"],
    Cell[8013, 275, 96, 2, 29, "Output",
      CellTags->"rw:3"],
    Cell[8124, 280, 141, 6, 33, "Text",
      CellTags->"rw:3"],
    Cell[8290, 290, 186, 4, 50, "Input",
      CellTags->"rw:3"],
    Cell[8479, 296, 104, 2, 29, "Output",
      CellTags->"rw:3"],
    Cell[8598, 301, 271, 8, 52, "Text",
      CellTags->"rw:3"],
    Cell[8872, 311, 356, 7, 90, "Input",
      CellTags->"rw:3"],
    Cell[9231, 320, 86, 1, 33, "Text",
      CellTags->"rw:3"],
    Cell[9342, 325, 63, 2, 30, "Input",
      CellTags->"rw:3"],
    Cell[9408, 329, 188, 4, 48, "Output",
      CellTags->"rw:3"]},
  "rw:4"->{
    Cell[1776, 53, 101, 1, 73, "Section",
      CellTags->{"rw:1", "rw:2", "rw:3", "rw:4", "rw:5", "rw:6", "rw:7"}],
    Cell[9645, 339, 86, 1, 38, "Subsection",
      CellTags->"rw:4"],
    Cell[9734, 342, 226, 6, 52, "Text",
      CellTags->"rw:4"],
    Cell[9963, 350, 168, 4, 50, "Input",
      CellTags->"rw:4"],
    Cell[10134, 356, 1390, 41, 185, "Text",
      CellTags->"rw:4"],
    Cell[11527, 399, 95, 1, 33, "Text",
      CellTags->"rw:4"],
    Cell[11647, 404, 75, 2, 30, "Input",
      CellTags->"rw:4"],
    Cell[11725, 408, 150, 4, 29, "Output",
      CellTags->"rw:4"],
    Cell[11890, 415, 392, 7, 90, "Text",
      CellTags->"rw:4"],
    Cell[12285, 424, 238, 7, 52, "Text",
      CellTags->"rw:4"],
    Cell[12548, 435, 70, 2, 30, "Input",
      CellTags->"rw:4"],
    Cell[12621, 439, 188, 4, 48, "Output",
      CellTags->"rw:4"],
    Cell[12824, 446, 736, 22, 109, "Text",
      CellTags->"rw:4"],
    Cell[13563, 470, 120, 4, 59, "Program",
      CellTags->"rw:4"],
    Cell[13686, 476, 398, 7, 90, "Text",
      CellTags->"rw:4"],
    Cell[14087, 485, 1216, 41, 109, "Text",
      CellTags->"rw:4"],
    Cell[15328, 530, 93, 2, 30, "Input",
      CellTags->"rw:4"],
    Cell[15424, 534, 75, 2, 29, "Output",
      CellTags->"rw:4"],
    Cell[15514, 539, 105, 3, 33, "Text",
      CellTags->"rw:4"],
    Cell[15622, 544, 322, 7, 90, "Input",
      CellTags->"rw:4"],
    Cell[15947, 553, 848, 27, 90, "Text",
      CellTags->"rw:4"],
    Cell[16798, 582, 205, 5, 52, "Text",
      CellTags->"rw:4"],
    Cell[17028, 591, 168, 5, 70, "Input",
      CellTags->"rw:4"],
    Cell[17199, 598, 155, 4, 29, "Output",
      CellTags->"rw:4"],
    Cell[17369, 605, 607, 18, 90, "Text",
      CellTags->"rw:4"]},
  "rw:5"->{
    Cell[1776, 53, 101, 1, 73, "Section",
      CellTags->{"rw:1", "rw:2", "rw:3", "rw:4", "rw:5", "rw:6", "rw:7"}],
    Cell[18013, 628, 75, 1, 38, "Subsection",
      CellTags->"rw:5"],
    Cell[18091, 631, 306, 7, 71, "Text",
      CellTags->"rw:5"],
    Cell[18400, 640, 58, 1, 30, "Input",
      CellTags->"rw:5"],
    Cell[18483, 645, 75, 2, 30, "Input",
      CellTags->"rw:5"],
    Cell[18561, 649, 417, 8, 124, "Output",
      CellTags->"rw:5"],
    Cell[18993, 660, 86, 1, 33, "Text",
      CellTags->"rw:5"],
    Cell[19082, 663, 204, 5, 50, "Input",
      CellTags->"rw:5"],
    Cell[19311, 672, 62, 2, 30, "Input",
      CellTags->"rw:5"],
    Cell[19376, 676, 249, 5, 85, "Output",
      CellTags->"rw:5"],
    Cell[19640, 684, 169, 5, 33, "Text",
      CellTags->"rw:5"],
    Cell[19812, 691, 170, 4, 50, "Input",
      CellTags->"rw:5"],
    Cell[20007, 699, 77, 2, 30, "Input",
      CellTags->"rw:5"],
    Cell[20087, 703, 152, 4, 29, "Output",
      CellTags->"rw:5"],
    Cell[20254, 710, 95, 1, 33, "Text",
      CellTags->"rw:5"],
    Cell[20352, 713, 233, 5, 85, "Input",
      CellTags->"rw:5"],
    Cell[20610, 722, 83, 2, 30, "Input",
      CellTags->"rw:5"],
    Cell[20696, 726, 152, 4, 29, "Output",
      CellTags->"rw:5"]},
  "rw:6"->{
    Cell[1776, 53, 101, 1, 73, "Section",
      CellTags->{"rw:1", "rw:2", "rw:3", "rw:4", "rw:5", "rw:6", "rw:7"}],
    Cell[20897, 736, 69, 1, 38, "Subsection",
      CellTags->"rw:6"],
    Cell[20969, 739, 520, 9, 109, "Text",
      CellTags->"rw:6"],
    Cell[21492, 750, 380, 8, 71, "Text",
      CellTags->"rw:6"],
    Cell[21875, 760, 437, 11, 71, "Text",
      CellTags->"rw:6"],
    Cell[22337, 775, 100, 2, 30, "Input",
      CellTags->"rw:6"],
    Cell[22440, 779, 139, 3, 29, "Output",
      CellTags->"rw:6"],
    Cell[22594, 785, 427, 9, 90, "Text",
      CellTags->"rw:6"],
    Cell[23024, 796, 204, 5, 52, "Text",
      CellTags->"rw:6"],
    Cell[23253, 805, 156, 4, 31, "Input",
      CellTags->"rw:6"],
    Cell[23412, 811, 90, 2, 29, "Output",
      CellTags->"rw:6"],
    Cell[23517, 816, 525, 13, 90, "Text",
      CellTags->"rw:6"],
    Cell[24045, 831, 588, 18, 90, "Text",
      CellTags->"rw:6"],
    Cell[24658, 853, 131, 3, 30, "Input",
      CellTags->"rw:6"],
    Cell[24792, 858, 101, 2, 29, "Output",
      CellTags->"rw:6"],
    Cell[24908, 863, 189, 5, 52, "Text",
      CellTags->"rw:6"],
    Cell[25122, 872, 68, 2, 30, "Input",
      CellTags->"rw:6"],
    Cell[25802, 896, 456, 11, 71, "Text",
      CellTags->"rw:6"],
    Cell[26261, 909, 167, 6, 42, "Program",
      Evaluatable->False,
      CellTags->"rw:6"],
    Cell[26431, 917, 375, 11, 52, "Text",
      CellTags->"rw:6"],
    Cell[26809, 930, 445, 11, 71, "Text",
      CellTags->"rw:6"],
    Cell[27257, 943, 310, 12, 96, "Program",
      CellTags->"rw:6"],
    Cell[27570, 957, 151, 5, 33, "Text",
      CellTags->"rw:6"],
    Cell[27724, 964, 58, 1, 30, "Input",
      CellTags->"rw:6"],
    Cell[27785, 967, 1400, 29, 374, "Input",
      CellTags->"rw:6"],
    Cell[29188, 998, 428, 13, 71, "Text",
      CellTags->"rw:6"],
    Cell[29619, 1013, 191, 7, 33, "Text",
      CellTags->"rw:6"],
    Cell[29835, 1024, 66, 2, 30, "Input",
      CellTags->"rw:6"],
    Cell[29904, 1028, 126, 3, 29, "Output",
      CellTags->"rw:6"],
    Cell[30045, 1034, 69, 1, 33, "Text",
      CellTags->"rw:6"],
    Cell[30139, 1039, 93, 2, 30, "Input",
      CellTags->"rw:6"],
    Cell[30235, 1043, 286, 6, 48, "Output",
      CellTags->"rw:6"],
    Cell[30536, 1052, 69, 1, 33, "Text",
      CellTags->"rw:6"],
    Cell[30630, 1057, 87, 2, 30, "Input",
      CellTags->"rw:6"],
    Cell[30720, 1061, 143, 3, 29, "Output",
      CellTags->"rw:6"],
    Cell[30878, 1067, 258, 6, 52, "Text",
      CellTags->"rw:6"],
    Cell[31139, 1075, 466, 9, 110, "Input",
      CellTags->"rw:6"],
    Cell[31608, 1086, 86, 1, 33, "Text",
      CellTags->"rw:6"],
    Cell[31719, 1091, 144, 4, 31, "Input",
      CellTags->"rw:6"],
    Cell[31866, 1097, 150, 4, 29, "Output",
      CellTags->"rw:6"],
    Cell[32053, 1106, 143, 4, 31, "Input",
      CellTags->"rw:6"],
    Cell[32199, 1112, 152, 4, 29, "Output",
      CellTags->"rw:6"],
    Cell[32388, 1121, 198, 5, 52, "Input",
      CellTags->"rw:6"],
    Cell[32589, 1128, 150, 4, 29, "Output",
      CellTags->"rw:6"]},
  "rw:7"->{
    Cell[1776, 53, 101, 1, 73, "Section",
      CellTags->{"rw:1", "rw:2", "rw:3", "rw:4", "rw:5", "rw:6", "rw:7"}],
    Cell[32788, 1138, 77, 1, 38, "Subsection",
      CellTags->"rw:7"],
    Cell[32868, 1141, 607, 12, 178, "Program",
      CellTags->"rw:7"]},
  "Info3287377721-8917883"->{
    Cell[25193, 876, 72, 1, 25, "Print",
      CellTags->"Info3287377721-8917883"],
    Cell[25268, 879, 519, 14, 25, "Print",
      CellTags->"Info3287377721-8917883"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"rw:1", 34126, 1173},
  {"rw:2", 34455, 1182},
  {"rw:3", 35903, 1225},
  {"rw:4", 36890, 1254},
  {"rw:5", 38628, 1305},
  {"rw:6", 39895, 1342},
  {"rw:7", 42953, 1432},
  {"Info3287377721-8917883", 43245, 1439}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 101, 1, 73, "Section",
  CellTags->{"rw:1", "rw:2", "rw:3", "rw:4", "rw:5", "rw:6", "rw:7"}],
Cell[1880, 56, 253, 7, 68, "Text"],

Cell[CellGroupData[{
Cell[2158, 67, 54, 1, 38, "Subsection",
  CellTags->"rw:1"],
Cell[2215, 70, 647, 14, 109, "Text",
  CellTags->"rw:1"],
Cell[2865, 86, 308, 6, 71, "Text",
  CellTags->"rw:1"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3210, 97, 73, 1, 38, "Subsection",
  CellTags->"rw:2"],
Cell[3286, 100, 950, 27, 109, "Text",
  CellTags->"rw:2"],

Cell[CellGroupData[{
Cell[4261, 131, 95, 2, 31, "Input",
  CellTags->"rw:2"],
Cell[4359, 135, 104, 2, 29, "Output",
  CellTags->"rw:2"]
}, Open  ]],
Cell[4478, 140, 333, 9, 52, "Text",
  CellTags->"rw:2"],

Cell[CellGroupData[{
Cell[4836, 153, 73, 2, 30, "Input",
  CellTags->"rw:2"],
Cell[4912, 157, 87, 2, 29, "Output",
  CellTags->"rw:2"]
}, Open  ]],
Cell[5014, 162, 262, 8, 52, "Text",
  CellTags->"rw:2"],
Cell[5279, 172, 136, 4, 31, "Input",
  CellTags->"rw:2"],
Cell[5418, 178, 153, 5, 33, "Text",
  CellTags->"rw:2"],

Cell[CellGroupData[{
Cell[5596, 187, 63, 2, 30, "Input",
  CellTags->"rw:2"],
Cell[5662, 191, 139, 3, 29, "Output",
  CellTags->"rw:2"]
}, Open  ]],
Cell[5816, 197, 179, 4, 52, "Text",
  CellTags->"rw:2"],
Cell[5998, 203, 126, 3, 31, "Input",
  CellTags->"rw:2"],
Cell[6127, 208, 92, 1, 33, "Text",
  CellTags->"rw:2"],

Cell[CellGroupData[{
Cell[6244, 213, 152, 3, 50, "Input",
  CellTags->"rw:2"],
Cell[6399, 218, 124, 3, 29, "Output",
  CellTags->"rw:2"]
}, Open  ]],
Cell[6538, 224, 132, 5, 33, "Text",
  CellTags->"rw:2"],

Cell[CellGroupData[{
Cell[6695, 233, 100, 2, 30, "Input",
  CellTags->"rw:2"],
Cell[6798, 237, 150, 4, 29, "Output",
  CellTags->"rw:2"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[6997, 247, 74, 1, 38, "Subsection",
  CellTags->"rw:3"],
Cell[7074, 250, 390, 7, 90, "Text",
  CellTags->"rw:3"],
Cell[7467, 259, 416, 8, 90, "Text",
  CellTags->"rw:3"],

Cell[CellGroupData[{
Cell[7908, 271, 102, 2, 30, "Input",
  CellTags->"rw:3"],
Cell[8013, 275, 96, 2, 29, "Output",
  CellTags->"rw:3"]
}, Open  ]],
Cell[8124, 280, 141, 6, 33, "Text",
  CellTags->"rw:3"],

Cell[CellGroupData[{
Cell[8290, 290, 186, 4, 50, "Input",
  CellTags->"rw:3"],
Cell[8479, 296, 104, 2, 29, "Output",
  CellTags->"rw:3"]
}, Open  ]],
Cell[8598, 301, 271, 8, 52, "Text",
  CellTags->"rw:3"],
Cell[8872, 311, 356, 7, 90, "Input",
  CellTags->"rw:3"],
Cell[9231, 320, 86, 1, 33, "Text",
  CellTags->"rw:3"],

Cell[CellGroupData[{
Cell[9342, 325, 63, 2, 30, "Input",
  CellTags->"rw:3"],
Cell[9408, 329, 188, 4, 48, "Output",
  CellTags->"rw:3"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[9645, 339, 86, 1, 38, "Subsection",
  CellTags->"rw:4"],
Cell[9734, 342, 226, 6, 52, "Text",
  CellTags->"rw:4"],
Cell[9963, 350, 168, 4, 50, "Input",
  CellTags->"rw:4"],
Cell[10134, 356, 1390, 41, 185, "Text",
  CellTags->"rw:4"],
Cell[11527, 399, 95, 1, 33, "Text",
  CellTags->"rw:4"],

Cell[CellGroupData[{
Cell[11647, 404, 75, 2, 30, "Input",
  CellTags->"rw:4"],
Cell[11725, 408, 150, 4, 29, "Output",
  CellTags->"rw:4"]
}, Open  ]],
Cell[11890, 415, 392, 7, 90, "Text",
  CellTags->"rw:4"],
Cell[12285, 424, 238, 7, 52, "Text",
  CellTags->"rw:4"],

Cell[CellGroupData[{
Cell[12548, 435, 70, 2, 30, "Input",
  CellTags->"rw:4"],
Cell[12621, 439, 188, 4, 48, "Output",
  CellTags->"rw:4"]
}, Open  ]],
Cell[12824, 446, 736, 22, 109, "Text",
  CellTags->"rw:4"],
Cell[13563, 470, 120, 4, 59, "Program",
  CellTags->"rw:4"],
Cell[13686, 476, 398, 7, 90, "Text",
  CellTags->"rw:4"],
Cell[14087, 485, 1216, 41, 109, "Text",
  CellTags->"rw:4"],

Cell[CellGroupData[{
Cell[15328, 530, 93, 2, 30, "Input",
  CellTags->"rw:4"],
Cell[15424, 534, 75, 2, 29, "Output",
  CellTags->"rw:4"]
}, Open  ]],
Cell[15514, 539, 105, 3, 33, "Text",
  CellTags->"rw:4"],
Cell[15622, 544, 322, 7, 90, "Input",
  CellTags->"rw:4"],
Cell[15947, 553, 848, 27, 90, "Text",
  CellTags->"rw:4"],
Cell[16798, 582, 205, 5, 52, "Text",
  CellTags->"rw:4"],

Cell[CellGroupData[{
Cell[17028, 591, 168, 5, 70, "Input",
  CellTags->"rw:4"],
Cell[17199, 598, 155, 4, 29, "Output",
  CellTags->"rw:4"]
}, Open  ]],
Cell[17369, 605, 607, 18, 90, "Text",
  CellTags->"rw:4"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18013, 628, 75, 1, 38, "Subsection",
  CellTags->"rw:5"],
Cell[18091, 631, 306, 7, 71, "Text",
  CellTags->"rw:5"],
Cell[18400, 640, 58, 1, 30, "Input",
  CellTags->"rw:5"],

Cell[CellGroupData[{
Cell[18483, 645, 75, 2, 30, "Input",
  CellTags->"rw:5"],
Cell[18561, 649, 417, 8, 124, "Output",
  CellTags->"rw:5"]
}, Open  ]],
Cell[18993, 660, 86, 1, 33, "Text",
  CellTags->"rw:5"],
Cell[19082, 663, 204, 5, 50, "Input",
  CellTags->"rw:5"],

Cell[CellGroupData[{
Cell[19311, 672, 62, 2, 30, "Input",
  CellTags->"rw:5"],
Cell[19376, 676, 249, 5, 85, "Output",
  CellTags->"rw:5"]
}, Open  ]],
Cell[19640, 684, 169, 5, 33, "Text",
  CellTags->"rw:5"],
Cell[19812, 691, 170, 4, 50, "Input",
  CellTags->"rw:5"],

Cell[CellGroupData[{
Cell[20007, 699, 77, 2, 30, "Input",
  CellTags->"rw:5"],
Cell[20087, 703, 152, 4, 29, "Output",
  CellTags->"rw:5"]
}, Open  ]],
Cell[20254, 710, 95, 1, 33, "Text",
  CellTags->"rw:5"],
Cell[20352, 713, 233, 5, 85, "Input",
  CellTags->"rw:5"],

Cell[CellGroupData[{
Cell[20610, 722, 83, 2, 30, "Input",
  CellTags->"rw:5"],
Cell[20696, 726, 152, 4, 29, "Output",
  CellTags->"rw:5"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[20897, 736, 69, 1, 38, "Subsection",
  CellTags->"rw:6"],
Cell[20969, 739, 520, 9, 109, "Text",
  CellTags->"rw:6"],
Cell[21492, 750, 380, 8, 71, "Text",
  CellTags->"rw:6"],
Cell[21875, 760, 437, 11, 71, "Text",
  CellTags->"rw:6"],

Cell[CellGroupData[{
Cell[22337, 775, 100, 2, 30, "Input",
  CellTags->"rw:6"],
Cell[22440, 779, 139, 3, 29, "Output",
  CellTags->"rw:6"]
}, Open  ]],
Cell[22594, 785, 427, 9, 90, "Text",
  CellTags->"rw:6"],
Cell[23024, 796, 204, 5, 52, "Text",
  CellTags->"rw:6"],

Cell[CellGroupData[{
Cell[23253, 805, 156, 4, 31, "Input",
  CellTags->"rw:6"],
Cell[23412, 811, 90, 2, 29, "Output",
  CellTags->"rw:6"]
}, Open  ]],
Cell[23517, 816, 525, 13, 90, "Text",
  CellTags->"rw:6"],
Cell[24045, 831, 588, 18, 90, "Text",
  CellTags->"rw:6"],

Cell[CellGroupData[{
Cell[24658, 853, 131, 3, 30, "Input",
  CellTags->"rw:6"],
Cell[24792, 858, 101, 2, 29, "Output",
  CellTags->"rw:6"]
}, Open  ]],
Cell[24908, 863, 189, 5, 52, "Text",
  CellTags->"rw:6"],

Cell[CellGroupData[{
Cell[25122, 872, 68, 2, 30, "Input",
  CellTags->"rw:6"],
Cell[25193, 876, 72, 1, 25, "Print",
  CellTags->"Info3287377721-8917883"],
Cell[25268, 879, 519, 14, 25, "Print",
  CellTags->"Info3287377721-8917883"]
}, Open  ]],
Cell[25802, 896, 456, 11, 71, "Text",
  CellTags->"rw:6"],
Cell[26261, 909, 167, 6, 42, "Program",
  Evaluatable->False,
  CellTags->"rw:6"],
Cell[26431, 917, 375, 11, 52, "Text",
  CellTags->"rw:6"],
Cell[26809, 930, 445, 11, 71, "Text",
  CellTags->"rw:6"],
Cell[27257, 943, 310, 12, 96, "Program",
  CellTags->"rw:6"],
Cell[27570, 957, 151, 5, 33, "Text",
  CellTags->"rw:6"],
Cell[27724, 964, 58, 1, 30, "Input",
  CellTags->"rw:6"],
Cell[27785, 967, 1400, 29, 374, "Input",
  CellTags->"rw:6"],
Cell[29188, 998, 428, 13, 71, "Text",
  CellTags->"rw:6"],
Cell[29619, 1013, 191, 7, 33, "Text",
  CellTags->"rw:6"],

Cell[CellGroupData[{
Cell[29835, 1024, 66, 2, 30, "Input",
  CellTags->"rw:6"],
Cell[29904, 1028, 126, 3, 29, "Output",
  CellTags->"rw:6"]
}, Open  ]],
Cell[30045, 1034, 69, 1, 33, "Text",
  CellTags->"rw:6"],

Cell[CellGroupData[{
Cell[30139, 1039, 93, 2, 30, "Input",
  CellTags->"rw:6"],
Cell[30235, 1043, 286, 6, 48, "Output",
  CellTags->"rw:6"]
}, Open  ]],
Cell[30536, 1052, 69, 1, 33, "Text",
  CellTags->"rw:6"],

Cell[CellGroupData[{
Cell[30630, 1057, 87, 2, 30, "Input",
  CellTags->"rw:6"],
Cell[30720, 1061, 143, 3, 29, "Output",
  CellTags->"rw:6"]
}, Open  ]],
Cell[30878, 1067, 258, 6, 52, "Text",
  CellTags->"rw:6"],
Cell[31139, 1075, 466, 9, 110, "Input",
  CellTags->"rw:6"],
Cell[31608, 1086, 86, 1, 33, "Text",
  CellTags->"rw:6"],

Cell[CellGroupData[{
Cell[31719, 1091, 144, 4, 31, "Input",
  CellTags->"rw:6"],
Cell[31866, 1097, 150, 4, 29, "Output",
  CellTags->"rw:6"]
}, Open  ]],

Cell[CellGroupData[{
Cell[32053, 1106, 143, 4, 31, "Input",
  CellTags->"rw:6"],
Cell[32199, 1112, 152, 4, 29, "Output",
  CellTags->"rw:6"]
}, Open  ]],

Cell[CellGroupData[{
Cell[32388, 1121, 198, 5, 52, "Input",
  CellTags->"rw:6"],
Cell[32589, 1128, 150, 4, 29, "Output",
  CellTags->"rw:6"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[32788, 1138, 77, 1, 38, "Subsection",
  CellTags->"rw:7"],
Cell[32868, 1141, 607, 12, 178, "Program",
  CellTags->"rw:7"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

